--[[----------------------------------
--  MessageFrame widget for AceGUI-3.0
--  Written by Shirokuma
--]]----------------------------------


--[[-----------------
-- AceGUI
--]]-----------------
local AceGUI = LibStub("AceGUI-3.0")

--[[-----------------
-- Lua APIs
--]]-----------------
local format, pairs, tostring = string.format, pairs, tostring

--[[-----------------
-- WoW APIs
--]]-----------------
local CreateFrame, UIParent = CreateFrame, UIParent

--[[-----------------
-- Frame Elements
--]]-----------------
local FrameBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}


--[[-----------------
-- Widget Info
--]]-----------------
local widgetType = "MessageFrame"
local widgetVersion = 1


--[[-----------------
-- Event Code
--]]-----------------
local function OnSizeChanged(self, width, height)
	self.obj.content:SetWidth(width)
	if self.obj.text then
		self.obj.text:SetWidth(width)
		self.obj.content:SetHeight(self.obj.text:GetHeight())
	end
end


--[[-----------------
-- MessageFrame Code
--]]-----------------
do
	local methods = {
		["OnAcquire"] = function (self)
			self.frame:SetWidth(400)
			self.frame:SetHeight(300)
		end,
		
		["SetText"] = function (self, txt)
			if self.text then
				self.text:SetText(txt)
				self.text:SetWidth(self.content:GetWidth())
				self.content:SetHeight(self.text:GetHeight())
			else
				local text = self.content:CreateFontString(nil,"OVERLAY","ChatFontNormal")
					text:SetJustifyH("LEFT")
					text:SetJustifyV("TOP")
					text:SetAllPoints()
					text:SetWidth(self.content:GetWidth())
					text:SetText(txt)
					self.text = text
					
					self.content:SetHeight(text:GetHeight())
			end
		end,
		
		["GetText"] = function (self)
			return self.text:GetText()
		end,
		
		["SetLabel"] = function (self, txt)
			self.label:SetText(txt)
		end,
	}

	local function Constructor()  -- widget constructor
		local frame = CreateFrame("Frame", nil, UIParent)
		local self = {}
		
		self.type = widgetType
		self.frame = frame
		frame.obj = self
		
		local label = frame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
			label:SetJustifyH("LEFT")
			label:SetPoint("TOPLEFT", 0, 0)
			label:SetPoint("TOPRIGHT", 0, 0)
			label:SetHeight(10)
			label:SetText(ACCEPT)
			self.label = label
		
		local backdrop = CreateFrame("Frame", nil, frame)
			backdrop:SetBackdrop(FrameBackdrop)
			backdrop:SetBackdropColor(0, 0, 0)
			backdrop:SetBackdropBorderColor(0.4, 0.4, 0.4)
			backdrop:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -10)
			backdrop:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -16, 0)
			self.backdrop = backdrop
		
		local scrollframe = CreateFrame("ScrollFrame", format("%s@%s@%s", widgetType, "ScrollFrame", tostring(self)), frame, "UIPanelScrollFrameTemplate")
		
			local scrollbar = _G[scrollframe:GetName() .. "ScrollBar"]
				scrollbar:ClearAllPoints()
				scrollbar:SetPoint("TOP", label, "BOTTOM", 0, -19)
				scrollbar:SetPoint("BOTTOM", frame, "BOTTOM", 0, 19)
				scrollbar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		
			scrollframe:SetPoint("TOPLEFT", backdrop, "TOPLEFT", 10, -6)
			scrollframe:SetPoint("BOTTOMRIGHT", backdrop, "BOTTOMRIGHT", -10, 6)
			scrollframe:SetScript("OnSizeChanged", OnSizeChanged)
			scrollframe.obj = self
			self.scrollframe = scrollframe

		local content = CreateFrame("Frame", nil, scrollframe)
			content:SetAllPoints()
			content.obj = self
			scrollframe:SetScrollChild(content)
			self.content = content
			
		for method, value in pairs (methods) do
			self[method] = value
		end
								
		AceGUI:RegisterAsWidget(self)
		return self
	end
	AceGUI:RegisterWidgetType(widgetType, Constructor, widgetVersion)
end